@@ -1,5 +1,5 @@
 /* communications.c -- Send commands to a running copy of ratpoison.
- * Copyright (C) 2000, 2001 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
  *
  * This file is part of ratpoison.
  *
@@ -32,52 +32,90 @@
 
 /* Sending commands to ratpoison */
 static void
-recieve_command_result (Window w)
+receive_command_result (Window w)
 {
+  int status;
   Atom type_ret;
   int format_ret;
   unsigned long nitems;
   unsigned long bytes_after;
-  unsigned char *result;
+  unsigned char *result = NULL;
 
+  /* First, find out how big the property is. */
+  status = XGetWindowProperty (dpy, w, rp_command_result,
+			       0, 0, False, XA_STRING,
+			       &type_ret, &format_ret, &nitems, &bytes_after,
+			       &result);
 
-  if (XGetWindowProperty (dpy, w, rp_command_result,
-			  0, 0, False, XA_STRING,
-			  &type_ret, &format_ret, &nitems, &bytes_after,
-			  &result) == Success
-      &&
-      XGetWindowProperty (dpy, w, rp_command_result,
-			  0, (bytes_after / 4) + (bytes_after % 4 ? 1 : 0),
-			  True, XA_STRING, &type_ret, &format_ret, &nitems, 
-			  &bytes_after, &result) == Success)
+  /* Failed to retrieve property. */
+  if (status != Success || result == NULL)
     {
-      if (result && strlen (result))
-	{
-	  printf ("%s\n", result);
-	}
-      XFree (result);
+      PRINT_DEBUG (("failed to get command result length\n"));
+      return;
     }
+
+  /* XGetWindowProperty always allocates one extra byte even if
+     the property is zero length. */
+  XFree (result);
+
+  /* Now that we have the length of the message, we can get the
+     whole message. */
+  status = XGetWindowProperty (dpy, w, rp_command_result,
+			       0, (bytes_after / 4) + (bytes_after % 4 ? 1 : 0),
+			       True, XA_STRING, &type_ret, &format_ret, &nitems, 
+			       &bytes_after, &result);
+
+  /* Failed to retrieve property. */
+  if (status != Success || result == NULL)
+    {
+      PRINT_DEBUG (("failed to get command result\n"));
+      return;
+    }
+
+  /* If result is not the empty string, print it. */
+  if (strlen (result))
+    printf ("%s\n", result);
+
+  /* Free the result. */
+  XFree (result);
 }
 
 int
-send_command (unsigned char *cmd)
+send_command (unsigned char interactive, unsigned char *cmd, int screen_num)
 {
   Window w;
   int done = 0;
+  struct sbuf *s;
 
-  w = XCreateSimpleWindow (dpy, DefaultRootWindow (dpy),
-			   0, 0, 1, 1, 0, 0, 0);
+  s = sbuf_new(0);
+  sbuf_printf(s, "%c%s", interactive, cmd);
+
+
+  /* If the user specified a specific screen, then send the event to
+     that screen. */
+  if (screen_num >= 0)
+    {
+      w = XCreateSimpleWindow (dpy, RootWindow (dpy, screen_num),
+			       0, 0, 1, 1, 0, 0, 0);
+    }
+  else
+    {
+      w = XCreateSimpleWindow (dpy, DefaultRootWindow (dpy),
+			       0, 0, 1, 1, 0, 0, 0);
+    }
 
   /* Select first to avoid race condition */
   XSelectInput (dpy, w, PropertyChangeMask);
 
   XChangeProperty (dpy, w, rp_command, XA_STRING,
-		   8, PropModeReplace, cmd, strlen (cmd) + 1);
+		   8, PropModeReplace, sbuf_get(s), strlen (cmd) + 2);
 
   XChangeProperty (dpy, DefaultRootWindow (dpy), 
 		   rp_command_request, XA_WINDOW,
 		   8, PropModeAppend, (unsigned char *)&w, sizeof (Window));
 
+  sbuf_free (s);
+
   while (!done)
     {
       XEvent ev;
@@ -86,7 +124,7 @@ send_command (unsigned char *cmd)
       if (ev.xproperty.atom == rp_command_result 
 	  && ev.xproperty.state == PropertyNewValue)
 	{
-	  recieve_command_result(ev.xproperty.window);
+	  receive_command_result(ev.xproperty.window);
 	  done = 1;
 	}
     }
