@@ -1,5 +1,5 @@
 /* communications.h -- Send commands to a running copy of ratpoison.
- * Copyright (C) 2000, 2001 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
  *
  * This file is part of ratpoison.
  *
@@ -22,6 +22,6 @@
 #ifndef _RATPOISON_COMMUNICATIONS_H
 #define _RATPOISON_COMMUNICATIONS_H 1
 
-int send_command (unsigned char *cmd, int screen_num);
+int send_command (unsigned char interactive, unsigned char *cmd, int screen_num);
 
 #endif /* ! _RATPOISON_COMMUNICATIONS_H */
