@@ -1,5 +1,5 @@
 /* Ratpoison X events
- * Copyright (C) 2000, 2001 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
  *
  * This file is part of ratpoison.
  *
@@ -50,9 +50,22 @@ new_window (XCreateWindowEvent *e)
   if (e->override_redirect)
     return;
 
-  s = find_screen (e->parent);
   win = find_window (e->window);
 
+  /* In Xinerama mode, all windows have the same root, so check
+   * all Xinerama screens
+   */
+  if (rp_have_xinerama)
+    {
+      /* New windows belong to the current screen */
+      s = &screens[rp_current_screen];
+    }
+  else
+    {
+      s = find_screen (e->parent);
+    }
+  if (is_rp_window_for_screen(e->window, s)) return;
+
   if (s && win == NULL
       && e->window != s->key_window 
       && e->window != s->bar_window 
@@ -176,7 +189,33 @@ destroy_window (XDestroyWindowEvent *ev)
   win = find_window (ev->window);
   if (win == NULL) return;
 
-  ignore_badwindow++;
+  ignore_badwindow++;  
+
+  /* If, somehow, the window is not withdrawn before it is destroyed,
+     perform the necessary steps to withdraw the window before it is
+     unmanaged. */
+  if (win->state == IconicState)
+    {
+      PRINT_DEBUG (("Destroying Iconic Window (%s)\n", window_name (win)));
+      withdraw_window (win);
+    }
+  else if (win->state == NormalState)
+    {
+      rp_frame *frame;
+
+      PRINT_DEBUG (("Destroying Normal Window (%s)\n", window_name (win)));
+      frame = find_windows_frame (win);
+      if (frame)
+	{
+	  cleanup_frame (frame);
+	  if (frame->number == win->scr->current_frame) 
+	    set_active_frame (frame);
+	}
+      withdraw_window (win);
+    }
+
+  /* Now that the window is guaranteed to be in the unmapped window
+     list, we can safely stop managing it. */
   unmanage (win);
   ignore_badwindow--;
 }
@@ -193,7 +232,7 @@ configure_request (XConfigureRequestEvent *e)
     {
       if (e->value_mask & CWStackMode)
 	{
-	  if (e->detail == Above)
+	  if (e->detail == Above && win->state != WithdrawnState)
 	    {
 	      /* Depending on the rudeness level, actually map the
 		 window. */
@@ -205,7 +244,7 @@ configure_request (XConfigureRequestEvent *e)
 		  else if (find_windows_frame (win))
 		    goto_window (win);
 		}
-	      else
+	      else if (current_window() != win)
 		{
 		  if (win->transient)
 		    marked_message_printf (0, 0, MESSAGE_RAISE_TRANSIENT,
@@ -323,29 +362,16 @@ client_msg (XClientMessageEvent *ev)
 }
 
 static void
-grab_rat ()
+handle_key (KeySym ks, unsigned int mod, rp_screen *s)
 {
-  XGrabPointer (dpy, current_screen()->root, True, 0, 
-		GrabModeAsync, GrabModeAsync, 
-		None, current_screen()->rat, CurrentTime);
-}
-
-static void
-ungrab_rat ()
-{
-  XUngrabPointer (dpy, CurrentTime);
-}
-
-static void
-handle_key (rp_screen *s)
-{
-  char *keysym_name;
   rp_action *key_action;
-  int revert;			
-  Window fwin;			/* Window currently in focus */
-  KeySym keysym;		/* Key pressed */
-  unsigned int mod;		/* Modifiers */
-  int rat_grabbed = 0;
+  rp_keymap *map = find_keymap (TOP_KEYMAP);
+
+  if (map == NULL)
+    {
+      PRINT_ERROR (("Unable to find " TOP_KEYMAP " keymap\n"));
+      return;
+    }
 
   PRINT_DEBUG (("handling key...\n"));
 
@@ -357,24 +383,14 @@ handle_key (rp_screen *s)
   alarm (0);
   alarm_signalled = 0;
 
-  XGetInputFocus (dpy, &fwin, &revert);
-  XSetInputFocus (dpy, s->key_window, RevertToPointerRoot, CurrentTime);
+  /* Call the top level key pressed hook. */
+  hook_run (&rp_key_hook);
 
-  /* Change the mouse icon to indicate to the user we are waiting for
-     more keystrokes */
-  if (defaults.wait_for_key_cursor)
-    {
-      grab_rat();
-      rat_grabbed = 1;
-    }
+  PRINT_DEBUG (("handle_key\n"));
 
-  read_key (&keysym, &mod, NULL, 0, 1);
-
-  XSetInputFocus (dpy, fwin, revert, CurrentTime);
-  if (rat_grabbed)
-    ungrab_rat();
-
-  if ((key_action = find_keybinding (keysym, x11_mask_to_rp_mask (mod))))
+  /* Read a key and execute the command associated with it on the
+     default keymap. Ignore the key if it doesn't have a binding. */
+  if ((key_action = find_keybinding (ks, x11_mask_to_rp_mask (mod), map)))
     {
       char *result;
       result = command (1, key_action->data);
@@ -385,10 +401,7 @@ handle_key (rp_screen *s)
     }
   else
     {
-      /* No key match, notify user. */
-      keysym_name = keysym_to_string (keysym, x11_mask_to_rp_mask (mod));
-      marked_message_printf (0, 0, " %s unbound key ", keysym_name);
-      free (keysym_name);
+      PRINT_ERROR(("Impossible: No matching key\n"));
     }
 }
 
@@ -399,10 +412,13 @@ key_press (XEvent *ev)
   unsigned int modifier;
   KeySym ks;
 
-  s = find_screen (ev->xkey.root);
+  if (rp_have_xinerama)
+    s = current_screen();
+  else
+    s = find_screen (ev->xkey.root);
 
 #ifdef HIDE_MOUSE
-  XWarpPointer (dpy, None, s->root, 0, 0, 0, 0, s->root_attr.width - 2, s->root_attr.height - 2); 
+  XWarpPointer (dpy, None, s->root, 0, 0, 0, 0, s->left + s->width - 2, s->top + s->height - 2); 
 #endif
 
   if (!s) return;
@@ -410,21 +426,7 @@ key_press (XEvent *ev)
   modifier = ev->xkey.state;
   cook_keycode ( &ev->xkey, &ks, &modifier, NULL, 0, 1);
 
-  if (ks == prefix_key.sym && (x11_mask_to_rp_mask (modifier) == prefix_key.state))
-    {
-      handle_key (s);
-    }
-  else
-    { 
-      if (current_window())
-	{
-	  ignore_badwindow++;
-	  ev->xkey.window = current_window()->w;
-	  XSendEvent (dpy, current_window()->w, False, KeyPressMask, ev);
-	  XSync (dpy, False);
-	  ignore_badwindow--;
-	}
-    }
+  handle_key (ks, modifier, s);
 }
 
 /* Read a command off the window and execute it. Some commands return
@@ -469,7 +471,7 @@ execute_remote_command (Window w)
     }
 
   PRINT_DEBUG (("command: %s\n", req));
-  result = command (0, req);
+  result = command (req[0], &req[1]);
   XFree (req);
 
   return result;
@@ -637,7 +639,7 @@ focus_change (XFocusChangeEvent *ev)
   if (win != NULL)
     {
       PRINT_DEBUG (("Re-grabbing prefix key\n"));
-      grab_prefix_key (win->w);
+      grab_top_level_keys (win->w);
     }
 }
 
@@ -649,7 +651,7 @@ mapping_notify (XMappingEvent *ev)
   /* Remove the grab on the current prefix key */
   list_for_each_entry (cur,&rp_mapped_window,node)
     {
-      ungrab_prefix_key (cur->w);
+      ungrab_top_level_keys (cur->w);
     }
 
   switch (ev->request)
@@ -665,7 +667,7 @@ mapping_notify (XMappingEvent *ev)
   /* Add the grab on the current prefix key */
   list_for_each_entry (cur, &rp_mapped_window,node)
     {
-      grab_prefix_key (cur->w);
+      grab_top_level_keys (cur->w);
     }
 }
 
@@ -762,7 +764,7 @@ handle_signals ()
     {
       int i;
 
-      PRINT_DEBUG (("Alarm recieved.\n"));
+      PRINT_DEBUG (("Alarm received.\n"));
 
       /* Only hide the bar if it times out. */
       if (defaults.bar_timeout > 0)
@@ -787,9 +789,9 @@ handle_signals ()
 	      if (cur->status != 0)
 		marked_message_printf (0,0, " /bin/sh -c \"%s\" finished (%d) ", 
 				       cur->cmd, cur->status);
+	      list_del  (&cur->node);
 	      free (cur->cmd);
 	      free (cur);
-	      list_del  (&cur->node);
 	    }
 	}
 
@@ -798,6 +800,8 @@ handle_signals ()
 
   if (hup_signalled > 0)
     {
+      PRINT_DEBUG (("Restarting\n"));
+      hook_run (&rp_restart_hook);
       clean_up (); 
       execvp(myargv[0], myargv);
     }
@@ -805,6 +809,7 @@ handle_signals ()
   if (kill_signalled > 0)
     {
       PRINT_DEBUG (("Exiting\n"));
+      hook_run (&rp_quit_hook);
       clean_up ();
       exit (EXIT_SUCCESS);
     }
@@ -845,3 +850,4 @@ listen_for_events ()
 	}
     }
 }
+
