@@ -1,5 +1,5 @@
 /* Read kdb input from the user.
- * Copyright (C) 2000, 2001 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
  *
  * This file is part of ratpoison.
  *
@@ -22,16 +22,13 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <unistd.h>
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 #include <X11/Xutil.h>
 
 #include "ratpoison.h"
 
-/* Variables to keep track of input history. */
-static char *input_history[INPUT_MAX_HISTORY];
-static int input_num_history_entries = 0;
-
 /* Convert an X11 modifier mask to the rp modifier mask equivalent, as
    best it can (the X server may not have a hyper key defined, for
    instance). */
@@ -42,6 +39,7 @@ x11_mask_to_rp_mask (unsigned int mask)
 
   PRINT_DEBUG (("x11 mask = %x\n", mask));
 
+  result |= mask & ShiftMask ? RP_SHIFT_MASK:0;
   result |= mask & ControlMask ? RP_CONTROL_MASK:0;
   result |= mask & rp_modifier_info.meta_mod_mask ? RP_META_MASK:0;
   result |= mask & rp_modifier_info.alt_mod_mask ? RP_ALT_MASK:0;
@@ -63,13 +61,14 @@ rp_mask_to_x11_mask (unsigned int mask)
 
   PRINT_DEBUG (("rp mask = %x\n", mask));
 
+  result |= mask & RP_SHIFT_MASK ? ShiftMask:0;
   result |= mask & RP_CONTROL_MASK ? ControlMask:0;
   result |= mask & RP_META_MASK ? rp_modifier_info.meta_mod_mask:0;
   result |= mask & RP_ALT_MASK ? rp_modifier_info.alt_mod_mask:0;
   result |= mask & RP_HYPER_MASK ? rp_modifier_info.hyper_mod_mask:0;
   result |= mask & RP_SUPER_MASK ? rp_modifier_info.super_mod_mask:0;
 
-  PRINT_DEBUG (("x11 mask = %x\n", mask));
+  PRINT_DEBUG (("x11 mask = %x\n", result));
 
   return result;
 }
@@ -164,16 +163,38 @@ update_modifier_map ()
   XFreeModifiermap (mods);
 }
 
+/* we need a keycode + modifier to generate the proper keysym (such as
+   @). */
+static void
+keysym_to_keycode_mod (KeySym keysym, KeyCode *code, unsigned int *mod)
+{
+  KeySym lower, upper;
+
+  *mod = 0;
+  *code = XKeysymToKeycode (dpy, keysym);
+  lower = XKeycodeToKeysym (dpy, *code, 0);
+  upper = XKeycodeToKeysym (dpy, *code, 1);
+  /* If you need to press shift to get the keysym, add the shift
+     mask. */
+  if (upper == keysym && lower != keysym)
+    *mod = ShiftMask;
+}
+
 /* Grab the key while ignoring annoying modifier keys including
    caps lock, num lock, and scroll lock. */
 void
-grab_key (int keycode, unsigned int modifiers, Window grab_window)
+grab_key (KeySym keysym, unsigned int modifiers, Window grab_window)
 {
   unsigned int mod_list[8];
   int i;
-
+  KeyCode keycode;
+  unsigned int mod;
+      
   /* Convert to a modifier mask that X Windows will understand. */
   modifiers = rp_mask_to_x11_mask (modifiers);
+  keysym_to_keycode_mod (keysym, &keycode, &mod);
+  PRINT_DEBUG (("keycode_mod: %ld %d %d\n", keysym, keycode, mod));
+  modifiers |= mod;
 
   /* Create a list of all possible combinations of ignored
      modifiers. Assumes there are only 3 ignored modifiers. */
@@ -205,11 +226,12 @@ keysym_to_string (KeySym keysym, unsigned int modifier)
 
   name = sbuf_new (0);
 
+  if (modifier & RP_SHIFT_MASK) sbuf_concat (name, "S-");
   if (modifier & RP_CONTROL_MASK) sbuf_concat (name, "C-");
   if (modifier & RP_META_MASK) sbuf_concat (name, "M-");
   if (modifier & RP_ALT_MASK) sbuf_concat (name, "A-");
   if (modifier & RP_HYPER_MASK) sbuf_concat (name, "H-");
-  if (modifier & RP_SUPER_MASK) sbuf_concat (name, "S-");
+  if (modifier & RP_SUPER_MASK) sbuf_concat (name, "s-");
     
   /* On solaris machines (perhaps other machines as well) this call
      can return NULL. In this case use the "NULL" string. */
@@ -237,6 +259,8 @@ int
 cook_keycode (XKeyEvent *ev, KeySym *keysym, unsigned int *mod, char *keysym_name, int len, int ignore_bad_mods)
 {
   int nbytes;
+  int shift = 0;
+  KeySym lower, upper;
  
   if (ignore_bad_mods)
     {
@@ -245,114 +269,165 @@ cook_keycode (XKeyEvent *ev, KeySym *keysym, unsigned int *mod, char *keysym_nam
 		     | rp_modifier_info.scroll_lock_mask);
     }
 
+  if (len > 0) len--;
   nbytes =  XLookupString (ev, keysym_name, len, keysym, NULL);
 
+  /* Null terminate the string (not all X servers do it for us). */
+  if (keysym_name) {
+    keysym_name[nbytes] = '\0';
+  }
+
+  /* Find out if XLookupString gobbled the shift modifier */
+  if (ev->state & ShiftMask)
+    {
+      lower = XKeycodeToKeysym (dpy, ev->keycode, 0);
+      upper = XKeycodeToKeysym (dpy, ev->keycode, 1);
+      /* If the keysym isn't affected by the shift key, then keep the
+	 shift modifier. */
+      if (lower == upper)
+	shift = ShiftMask;
+    }
+
   *mod = ev->state;
   *mod &= (rp_modifier_info.meta_mod_mask
 	   | rp_modifier_info.alt_mod_mask
 	   | rp_modifier_info.hyper_mod_mask
 	   | rp_modifier_info.super_mod_mask
-	   | ControlMask );
+	   | ControlMask
+	   | shift);
 
   return nbytes;
 }
 
 int
-read_key (KeySym *keysym, unsigned int *modifiers, char *keysym_name, int len, int gobble_rel)
+read_key (KeySym *keysym, unsigned int *modifiers, char *keysym_name, int len)
 {
   XEvent ev;
   int nbytes;
-  unsigned int keycode;
 
   /* Read a key from the keyboard. */
   do
     {
       XMaskEvent (dpy, KeyPressMask, &ev);
-      /* Store the keycode so we can wait for it's corresponding key
-	 release event. */
-      keycode = ev.xkey.keycode;
       *modifiers = ev.xkey.state;
       nbytes = cook_keycode (&ev.xkey, keysym, modifiers, keysym_name, len, 0);
     } while (IsModifierKey (*keysym));
 
-  /* Gobble the release event for the key we pressed. */
-  if (gobble_rel)
-    do
-      {
-	XMaskEvent (dpy, KeyReleaseMask, &ev);
-      } while (ev.xkey.keycode != keycode);
-
   return nbytes;
 }
 
 static void
-update_input_window (rp_screen *s, char *prompt, char *input, int input_len)
+update_input_window (rp_screen *s, rp_input_line *line)
 {
-  int 	prompt_width = XTextWidth (defaults.font, prompt, strlen (prompt));
-  int 	input_width  = XTextWidth (defaults.font, input, input_len);
-  int 	width, height;
+  int 	prompt_width = XTextWidth (defaults.font, line->prompt, strlen (line->prompt));
+  int 	input_width  = XTextWidth (defaults.font, line->buffer, line->length);
+  int 	total_width;
+  GC lgc;
+  XGCValues gv;
+  int height;
 
-  width = defaults.bar_x_padding * 2 + prompt_width + input_width;
+  total_width = defaults.bar_x_padding * 2 + prompt_width + input_width + MAX_FONT_WIDTH (defaults.font);
   height = (FONT_HEIGHT (defaults.font) + defaults.bar_y_padding * 2);
 
-  if (width < defaults.input_window_size + prompt_width)
+  if (total_width < defaults.input_window_size + prompt_width)
     {
-      width = defaults.input_window_size + prompt_width;
+      total_width = defaults.input_window_size + prompt_width;
     }
 
   XMoveResizeWindow (dpy, s->input_window, 
-		     bar_x (s, width), bar_y (s, height), width, height);
+		     bar_x (s, total_width), bar_y (s, height), total_width,
+ 		     (FONT_HEIGHT (defaults.font) + defaults.bar_y_padding * 2));
 
   XClearWindow (dpy, s->input_window);
   XSync (dpy, False);
 
   XDrawString (dpy, s->input_window, s->normal_gc, 
-	       defaults.bar_x_padding, 
-	       defaults.bar_y_padding + defaults.font->max_bounds.ascent, prompt, 
-	       strlen (prompt));
-
+ 	       defaults.bar_x_padding, 
+	       defaults.bar_y_padding + defaults.font->max_bounds.ascent,
+	       line->prompt, 
+	       strlen (line->prompt));
+ 
   XDrawString (dpy, s->input_window, s->normal_gc, 
-	       defaults.bar_x_padding + prompt_width,
-	       defaults.bar_y_padding + defaults.font->max_bounds.ascent, input, 
-	       input_len);
+ 	       defaults.bar_x_padding + prompt_width,
+	       defaults.bar_y_padding + defaults.font->max_bounds.ascent,
+	       line->buffer, 
+	       line->length);
 
-  /* Draw a cheap-o cursor. */
-  XDrawLine (dpy, s->input_window, s->normal_gc, 
-	     defaults.bar_x_padding + prompt_width + input_width + 2, 
-	     defaults.bar_y_padding + 1, 
-	     defaults.bar_x_padding + prompt_width + input_width + 2,
-	     defaults.bar_y_padding + FONT_HEIGHT (defaults.font) - 1);
+  gv.function = GXxor;
+  gv.foreground = s->fg_color ^ s->bg_color;
+  lgc = XCreateGC (dpy, s->input_window, GCFunction | GCForeground, &gv);
+
+  /* Draw a cheap-o cursor - MkII */
+  XFillRectangle (dpy, s->input_window, lgc, 
+		  defaults.bar_x_padding + prompt_width + XTextWidth (defaults.font, line->buffer, line->position),
+		  defaults.bar_y_padding, 
+		  XTextWidth (defaults.font, &line->buffer[line->position], 1),
+		  FONT_HEIGHT (defaults.font));
+
+  XFlush (dpy);
+  XFreeGC (dpy, lgc);
 }
 
-char *
-get_input (char *prompt)
+void
+ring_bell ()
 {
-  return get_more_input (prompt, "");
+#ifdef VISUAL_BELL
+  GC lgc;
+  XGCValues gv;
+  XWindowAttributes attr;
+  rp_screen *s = current_screen ();
+
+  XGetWindowAttributes (dpy, s->input_window, &attr);
+
+  gv.function = GXxor;
+  gv.foreground = s->fg_color ^ s->bg_color;
+  lgc = XCreateGC (dpy, s->input_window, GCFunction | GCForeground, &gv);
+
+  XFillRectangle (dpy, s->input_window, lgc, 0, 0, attr.width, attr.height);
+  XFlush (dpy);
+
+#ifdef HAVE_USLEEP
+  usleep (15000);
+#else
+  {
+    struct timeval tv;
+
+    tv.tv_sec = 0;
+    tv.tv_usec = 15000;
+    select (0, NULL, NULL, NULL, &tv);
+  }
+#endif
+  XFillRectangle (dpy, s->input_window, lgc, 0, 0, attr.width, attr.height);
+  XFlush (dpy);
+  XFreeGC (dpy, lgc);
+#else
+  XBell (dpy, 0);
+#endif
 }
 
 char *
-get_more_input (char *prompt, char *preinput)
+get_input (char *prompt, completion_fn fn)
+{
+  return get_more_input (prompt, "", fn);
+}
+
+char *
+get_more_input (char *prompt, char *preinput, 
+		completion_fn compl_fn)
 {
   /* Emacs 21 uses a 513 byte string to store the keysym name. */
   char keysym_buf[513];	
   int keysym_bufsize = sizeof (keysym_buf);
   int nbytes;
   rp_screen *s = current_screen ();
-  int cur_len = 0;		/* Current length of the string. */
-  int allocated_len=100; /* The amount of memory we allocated for str */
   KeySym ch;
   unsigned int modifier;
-  int revert;
-  Window fwin;
-  char *str;
-  int history_index = input_num_history_entries;
+  rp_input_line *line;
+  char *final_input;
+  edit_status status;
 
-  /* Allocate some memory to start with. */
-  str = (char *) xmalloc ( allocated_len );
-
-  /* load in the preinput */
-  strcpy (str, preinput);
-  cur_len = strlen (preinput);
+  /* Create our line structure */
+  line = input_line_new (prompt, preinput, compl_fn);
 
   /* We don't want to draw overtop of the program bar. */
   hide_bar (s);
@@ -362,117 +437,50 @@ get_more_input (char *prompt, char *preinput)
   XClearWindow (dpy, s->input_window);
   XSync (dpy, False);
 
-  update_input_window (s, prompt, str, cur_len);
+  update_input_window (s, line);
 
-  XGetInputFocus (dpy, &fwin, &revert);
-  XSetInputFocus (dpy, s->input_window, RevertToPointerRoot, CurrentTime);
-  /* XSync (dpy, False); */
+  XGrabKeyboard (dpy, s->input_window, False, GrabModeSync, GrabModeAsync, CurrentTime);
 
-
-  nbytes = read_key (&ch, &modifier, keysym_buf, keysym_bufsize, 0);
-  while (ch != XK_Return) 
+  for (;;)
     {
-      PRINT_DEBUG (("key %ld\n", ch));
-      if (ch == XK_BackSpace)
-	{
-	  if (cur_len > 0) cur_len--;
-	  update_input_window(s, prompt, str, cur_len);
-	}
-      else if (ch == INPUT_PREV_HISTORY_KEY 
-	       && modifier == INPUT_PREV_HISTORY_MODIFIER)
-	{
-	  /* Cycle through the history. */
-	  if (input_num_history_entries > 0)
-	    {
-	      history_index--;
-	      if (history_index < 0)
-		{
-		  history_index = input_num_history_entries - 1;
-		}
+      nbytes = read_key (&ch, &modifier, keysym_buf, keysym_bufsize);
+      modifier = x11_mask_to_rp_mask (modifier);
+      PRINT_DEBUG (("ch = %ld, modifier = %d, keysym_buf = %s, keysym_bufsize = %d\n", 
+		    ch, modifier, keysym_buf, keysym_bufsize));
+      status = execute_edit_action (line, ch, modifier, keysym_buf);
 
-	      free (str);
-	      str = xstrdup (input_history[history_index]);
-	      allocated_len = strlen (str) + 1;
-	      cur_len = allocated_len - 1;
-
-	      update_input_window (s, prompt, str, cur_len);
-	    }
-	}
-      else if (ch == INPUT_NEXT_HISTORY_KEY 
-	       && modifier == INPUT_NEXT_HISTORY_MODIFIER)
-	{
-	  /* Cycle through the history. */
-	  if (input_num_history_entries > 0)
-	    {
-	      history_index++;
-	      if (history_index >= input_num_history_entries)
-		{
-		  history_index = 0;
-		}
-
-	      free (str);
-	      str = xstrdup (input_history[history_index]);
-	      allocated_len = strlen (str) + 1;
-	      cur_len = allocated_len - 1;
-
-	      update_input_window (s, prompt, str, cur_len);
-	    }
-	}
-      else if (ch == INPUT_ABORT_KEY && modifier == INPUT_ABORT_MODIFIER)
-	{
-	  /* User aborted. */
-	  free (str);
-	  XSetInputFocus (dpy, fwin, RevertToPointerRoot, CurrentTime);
-	  XUnmapWindow (dpy, s->input_window);
-	  return NULL;
-	}
-      else
-	{
-	  if (cur_len + nbytes > allocated_len - 1)
-	    {
-	      allocated_len += nbytes + 100;
-	      str = xrealloc ( str, allocated_len );
-	    }
-
-	  strncpy (&str[cur_len], keysym_buf, nbytes);
-/* 	  str[cur_len] = ch; */
-	  cur_len+=nbytes;
-
-	  update_input_window(s, prompt, str, cur_len);
-	}
-
-      nbytes = read_key (&ch, &modifier, keysym_buf, keysym_bufsize, 0);
+      if (status == EDIT_DELETE || status == EDIT_INSERT || status == EDIT_MOVE
+	  || status == EDIT_COMPLETE)
+        {
+	  /* If the text changed (and we didn't just complete
+	     something) then set the virgin bit. */
+	  if (status != EDIT_COMPLETE)
+	    line->compl->virgin = 1;
+	  /* In all cases, we need to redisplay the input string. */
+          update_input_window (s, line);
+        }
+      else if (status == EDIT_NO_OP)
+        {
+          ring_bell ();
+        }
+      else if (status == EDIT_ABORT)
+        {
+          final_input = NULL;
+          break;
+        }
+      else if (status == EDIT_DONE)
+        {
+          final_input = xstrdup (line->buffer);
+          break;
+        }
     }
 
-  str[cur_len] = 0;
+  /* Clean up our line structure */
+  input_line_free (line);
 
-  /* Push the history entries down. */
-  if (input_num_history_entries >= INPUT_MAX_HISTORY)
-    {
-      int i;
-      free (input_history[0]);
-      for (i=0; i<INPUT_MAX_HISTORY-1; i++)
-	{
-	  input_history[i] = input_history[i+1];
-	}
-
-      input_num_history_entries--;
-    }
-
-  /* Store the string in the history. */
-  input_history[input_num_history_entries] = xstrdup (str);
-  input_num_history_entries++;
-
-  XSetInputFocus (dpy, fwin, RevertToPointerRoot, CurrentTime);
+  /* Revert focus. */
+  XUngrabKeyboard (dpy, CurrentTime);
   XUnmapWindow (dpy, s->input_window);
-  return str;
+
+  return final_input;
 }  
-
-void
-free_history ()
-{
-  int i;
-
-  for (i=0; i<input_num_history_entries; i++)
-    free (input_history[i]);
-}
