@@ -1,5 +1,5 @@
 /* Ratpoison.
- * Copyright (C) 2000, 2001 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
  *
  * This file is part of ratpoison.
  *
@@ -34,6 +34,7 @@
 #include <getopt.h>
 #include <string.h>
 #include <sys/wait.h>
+#include <ctype.h>
 
 #include "ratpoison.h"
 
@@ -43,62 +44,18 @@
 # define WAIT_ANY -1
 #endif
 
-static void init_screen (rp_screen *s, int screen_num);
-
-int alarm_signalled = 0;
-int kill_signalled = 0;
-int hup_signalled = 0;
-int chld_signalled = 0;
-int rat_x;
-int rat_y;
-int rat_visible = 1;		/* rat is visible by default */
-
-Atom wm_name;
-Atom wm_state;
-Atom wm_change_state;
-Atom wm_protocols;
-Atom wm_delete;
-Atom wm_take_focus;
-Atom wm_colormaps;
-
-Atom rp_command;
-Atom rp_command_request;
-Atom rp_command_result;
-
-int rp_current_screen;
-rp_screen *screens;
-int num_screens;
-Display *dpy;
-
-LIST_HEAD (rp_children);
-struct rp_defaults defaults;
-
-int ignore_badwindow = 0;
-
-char **myargv;
-
-struct rp_key prefix_key;
-
-struct modifier_info rp_modifier_info;
-
-/* rudeness levels */
-int rp_honour_transient_raise = 1;
-int rp_honour_normal_raise = 1;
-int rp_honour_transient_map = 1;
-int rp_honour_normal_map = 1;
-
-char *rp_error_msg = NULL;
-
 /* Command line options */
 static struct option ratpoison_longopts[] = 
-  { {"help", 	no_argument, 		0, 	'h'},
-    {"version", no_argument, 		0, 	'v'},
-    {"command", required_argument, 	0, 	'c'},
-    {"display", required_argument, 	0, 	'd'},
-    {"screen",  required_argument,      0,      's'},
-    {0, 	0, 			0, 	0} };
+  { {"help", 		no_argument, 		0, 	'h'},
+    {"interactive", 	no_argument, 		0, 	'i'},
+    {"version", 	no_argument, 		0, 	'v'},
+    {"command", 	required_argument, 	0, 	'c'},
+    {"display", 	required_argument, 	0, 	'd'},
+    {"screen",  	required_argument,      0,      's'},
+    {"file",  		required_argument,      0,      'f'},
+    {0, 		0, 			0, 	0} };
 
-static char ratpoison_opts[] = "hvc:d:s:";
+static char ratpoison_opts[] = "hvic:d:s:f:";
 
 void
 fatal (const char *msg)
@@ -122,7 +79,6 @@ xrealloc (void *ptr, size_t size)
   register void *value = realloc (ptr, size);
   if (value == 0)
     fatal ("Virtual memory exhausted");
-  PRINT_DEBUG (("realloc: %d\n", size));
   return value;
 }
 
@@ -195,6 +151,18 @@ xsprintf (char *fmt, ...)
   return buffer;
 }
 
+/* A case insensitive strncmp. */
+int
+str_comp (char *s1, char *s2, int len)
+{
+  int i;
+
+  for (i=0; i<len; i++)
+    if (toupper (s1[i]) != toupper (s2[i])) return 0;
+
+  return 1;
+}
+
 void
 sighandler (int signum)
 {
@@ -216,10 +184,9 @@ alrm_handler (int signum)
 void
 chld_handler (int signum)
 {
-  int pid, status, serrno;
   rp_child_info *cur;
+  int pid, status, serrno;
 
-  /* Save the errorno so we can restore it afterwards. */
   serrno = errno;
 
   while (1)
@@ -322,13 +289,24 @@ print_help ()
   printf ("-v, --version         Display the version\n");
   printf ("-d, --display <dpy>   Set the X display to use\n");
   printf ("-s, --screen <num>    Only use the specified screen\n");
-  printf ("-c, --command <cmd>   Send ratpoison a colon-command\n\n");
+  printf ("-c, --command <cmd>   Send ratpoison a colon-command\n");
+  printf ("-i, --interactive     Execute commands in interactive mode\n");
+  printf ("-f, --file <file>     Specify an alternative configuration file\n\n");
 
   printf ("Report bugs to ratpoison-devel@lists.sourceforge.net\n\n");
 
   exit (EXIT_SUCCESS);
 }
 
+void
+set_close_on_exec (FILE *fd)
+{
+  int fnum = fileno (fd);
+  int flags = fcntl (fnum, F_GETFD);
+  if (flags >= 0)
+    fcntl (fnum, F_SETFD, flags | FD_CLOEXEC);
+}
+
 void
 read_rc_file (FILE *file)
 {
@@ -385,43 +363,54 @@ read_rc_file (FILE *file)
 }
 
 static void
-read_startup_files ()
+read_startup_files (char *alt_rcfile)
 {
   char *homedir;
-  FILE *fileptr;
+  FILE *fileptr = NULL;
 
-  /* first check $HOME/.ratpoisonrc and if that does not exist then try
-     /etc/ratpoisonrc */
-
-  homedir = getenv ("HOME");
-  if (!homedir)
+  if (alt_rcfile)
     {
-      PRINT_ERROR (("ratpoison: $HOME not set!?\n"));
+      if ((fileptr = fopen (alt_rcfile, "r")) == NULL)
+	{
+	  /* we probably don't need to report this, its not an error */
+	  PRINT_DEBUG (("ratpoison: could not open %s\n", alt_rcfile));
+	}
     }
   else
     {
-      char *filename = (char*)xmalloc (strlen (homedir) + strlen ("/.ratpoisonrc") + 1);
-      sprintf (filename, "%s/.ratpoisonrc", homedir);
+      /* first check $HOME/.ratpoisonrc and if that does not exist then try
+	 /etc/ratpoisonrc */
+
+      homedir = getenv ("HOME");
+      if (!homedir)
+	{
+	  PRINT_ERROR (("ratpoison: $HOME not set!?\n"));
+	}
+      else
+	{
+	  char *filename = (char*)xmalloc (strlen (homedir) + strlen ("/.ratpoisonrc") + 1);
+	  sprintf (filename, "%s/.ratpoisonrc", homedir);
       
-      if ((fileptr = fopen (filename, "r")) == NULL)
-	{
-	  /* we probably don't need to report this, its not an error */
-	  PRINT_DEBUG (("ratpoison: could not open %s\n", filename));
-
-	  if ((fileptr = fopen ("/etc/ratpoisonrc", "r")) == NULL)
+	  if ((fileptr = fopen (filename, "r")) == NULL)
 	    {
-	      /* neither is this */
-	      PRINT_DEBUG (("ratpoison: could not open /etc/ratpoisonrc\n"));
+	      /* we probably don't need to report this, its not an error */
+	      PRINT_DEBUG (("ratpoison: could not open %s\n", filename));
+
+	      if ((fileptr = fopen ("/etc/ratpoisonrc", "r")) == NULL)
+		{
+		  /* neither is this */
+		  PRINT_DEBUG (("ratpoison: could not open /etc/ratpoisonrc\n"));
+		}
 	    }
+	  free (filename);
 	}
+    }
 
-      if (fileptr)
-	{
-	  read_rc_file (fileptr);
-	  fclose (fileptr);
-	}
-
-      free (filename);
+  if (fileptr)
+    {
+      set_close_on_exec(fileptr);
+      read_rc_file (fileptr);
+      fclose (fileptr);
     }
 }
 
@@ -432,11 +421,14 @@ show_welcome_message ()
 {
   rp_action *help_action;
   char *prefix, *help;
+  rp_keymap *map;
   
   prefix = keysym_to_string (prefix_key.sym, prefix_key.state);
 
+  map = find_keymap (ROOT_KEYMAP); 
+
   /* Find the help key binding. */
-  help_action = find_keybinding_by_action ("help");
+  help_action = find_keybinding_by_action ("help " ROOT_KEYMAP, map);
   if (help_action)
     help = keysym_to_string (help_action->key, help_action->state);
   else
@@ -491,6 +483,10 @@ init_defaults ()
       fprintf (stderr, "ratpoison: Cannot load font %s.\n", "9x15bold");
       exit (EXIT_FAILURE);
     }
+  defaults.font_string = xstrdup ("9x15bold");
+
+  defaults.fgcolor_string = xstrdup ("black");
+  defaults.bgcolor_string = xstrdup ("white");
 
   defaults.wait_for_key_cursor = 1;
 
@@ -499,7 +495,10 @@ init_defaults ()
   defaults.win_name = WIN_NAME_TITLE;
   defaults.startup_message = 1;
   defaults.warp = 1;
-  defaults.window_list_style = STYLE_ROW;
+  defaults.window_list_style = STYLE_COLUMN;
+
+  defaults.history_size = 20;
+  defaults.frame_selectors = xstrdup ("");
 }
 
 int
@@ -512,6 +511,8 @@ main (int argc, char *argv[])
   int screen_arg = 0;
   int screen_num = 0;
   char *display = NULL;
+  unsigned char interactive = 0;
+  char *alt_rcfile = NULL;
 
   myargv = argv;
 
@@ -552,6 +553,12 @@ main (int argc, char *argv[])
 	  screen_arg = 1;
 	  screen_num = strtol (optarg, NULL, 10);
 	  break;
+	case 'i':
+	  interactive = 1;
+	  break;
+	case 'f':
+	  alt_rcfile = xstrdup (optarg);
+	  break;
 	  
 	default:
 	  exit (EXIT_FAILURE);
@@ -568,6 +575,7 @@ main (int argc, char *argv[])
   rp_command = XInternAtom (dpy, "RP_COMMAND", False);
   rp_command_request = XInternAtom (dpy, "RP_COMMAND_REQUEST", False);
   rp_command_result = XInternAtom (dpy, "RP_COMMAND_RESULT", False);
+  rp_selection = XInternAtom (dpy, "RP_SELECTION", False);
 
   if (cmd_count > 0)
     {
@@ -576,9 +584,9 @@ main (int argc, char *argv[])
       for (i=0; i<cmd_count; i++)
 	{
 	  if (screen_arg)
-	    send_command (command[i], screen_num);
+	    send_command (interactive, command[i], screen_num);
 	  else
-	    send_command (command[i], -1);
+	    send_command (interactive, command[i], -1);
 
 	  free (command[i]);
 	}
@@ -607,43 +615,17 @@ main (int argc, char *argv[])
 
   /* Setup ratpoison's internal structures */
   init_defaults ();
+  init_groups ();
   init_window_stuff ();
-
-  /* Get the number of screens */
-  num_screens = ScreenCount (dpy);
-
-  /* make sure the screen specified is valid. */
-  if (screen_arg)
-    {
-      if (screen_num < 0 || screen_num >= num_screens)
-	{
-	  fprintf (stderr, "%d is an invalid screen for the display\n", screen_num);
-	  exit (EXIT_FAILURE);
-	}
-
-      /* we're only going to use one screen. */
-      num_screens = 1;
-    }
-
-  /* Initialize the screens */
-  screens = (rp_screen *)xmalloc (sizeof (rp_screen) * num_screens);
-  PRINT_DEBUG (("%d screens.\n", num_screens));
-
-  if (screen_arg)
-    {
-      init_screen (&screens[0], screen_num);
-    }
-  else
-    {
-      for (i=0; i<num_screens; i++)
-	{
-	  init_screen (&screens[i], i);
-	}
-    }
+  init_xinerama ();
+  init_screens (screen_arg, screen_num);
 
   init_frame_lists ();
   update_modifier_map ();
   initialize_default_keybindings ();
+#ifdef HAVE_READLINE_HISTORY_H
+  history_load ();
+#endif /* HAVE_READLINE_HISTORY_H */
 
   /* Scan for windows */
   if (screen_arg)
@@ -660,7 +642,9 @@ main (int argc, char *argv[])
 	}
     }
 
-  read_startup_files ();
+  read_startup_files (alt_rcfile);
+  if (alt_rcfile)
+    free (alt_rcfile);
 
   /* Indicate to the user that ratpoison has booted. */
   if (defaults.startup_message)
@@ -668,104 +652,13 @@ main (int argc, char *argv[])
 
   /* If no window has focus, give the key_window focus. */
   if (current_window() == NULL)
-    XSetInputFocus (dpy, current_screen()->key_window, 
-		    RevertToPointerRoot, CurrentTime);
+    set_window_focus (current_screen()->key_window);
 
   listen_for_events ();
 
   return EXIT_SUCCESS;
 }
 
-static void
-init_rat_cursor (rp_screen *s)
-{
-  s->rat = XCreateFontCursor( dpy, XC_icon );
-}
-
-static void
-init_screen (rp_screen *s, int screen_num)
-{
-  XGCValues gv;
-
-  /* Select on some events on the root window, if this fails, then
-     there is already a WM running and the X Error handler will catch
-     it, terminating ratpoison. */
-  XSelectInput(dpy, RootWindow (dpy, screen_num),
-	       PropertyChangeMask | ColormapChangeMask
-	       | SubstructureRedirectMask | SubstructureNotifyMask );
-  XSync (dpy, False);
-
-  /* Create the numset for the frames. */
-  s->frames_numset = numset_new ();
-
-  /* Build the display string for each screen */
-  s->display_string = xmalloc (strlen(DisplayString (dpy)) + 21);
-  sprintf (s->display_string, "DISPLAY=%s", DisplayString (dpy));
-  if (strrchr (DisplayString (dpy), ':'))
-    {
-      char *dot;
-
-      dot = strrchr(s->display_string, '.');
-      if (dot)
-	sprintf(dot, ".%i", screen_num);
-    }
-
-  PRINT_DEBUG (("%s\n", s->display_string));
-
-  s->screen_num = screen_num;
-  s->root = RootWindow (dpy, screen_num);
-  s->def_cmap = DefaultColormap (dpy, screen_num);
-  XGetWindowAttributes (dpy, s->root, &s->root_attr);
-  
-  init_rat_cursor (s);
-
-  s->fg_color = BlackPixel (dpy, s->screen_num);
-  s->bg_color = WhitePixel (dpy, s->screen_num);
-
-  /* Setup the GC for drawing the font. */
-  gv.foreground = s->fg_color;
-  gv.background = s->bg_color;
-  gv.function = GXcopy;
-  gv.line_width = 1;
-  gv.subwindow_mode = IncludeInferiors;
-  gv.font = defaults.font->fid;
-  s->normal_gc = XCreateGC(dpy, s->root, 
-			   GCForeground | GCBackground | GCFunction 
-			   | GCLineWidth | GCSubwindowMode | GCFont, 
-			   &gv);
-
-  /* Create the program bar window. */
-  s->bar_is_raised = 0;
-  s->bar_window = XCreateSimpleWindow (dpy, s->root, 0, 0, 1, 1, 
-				       defaults.bar_border_width, 
-				       s->fg_color, s->bg_color);
-
-  /* Setup the window that will recieve all keystrokes once the prefix
-     key has been pressed. */
-  s->key_window = XCreateSimpleWindow (dpy, s->root, 0, 0, 1, 1, 0, 
-				       WhitePixel (dpy, s->screen_num), 
-				       BlackPixel (dpy, s->screen_num));
-  XSelectInput (dpy, s->key_window, KeyPressMask | KeyReleaseMask);
-  XMapWindow (dpy, s->key_window);
-
-  /* Create the input window. */
-  s->input_window = XCreateSimpleWindow (dpy, s->root, 0, 0, 1, 1, 
-					 defaults.bar_border_width, 
-					 s->fg_color, s->bg_color);
-  XSelectInput (dpy, s->input_window, KeyPressMask | KeyReleaseMask);
-
-  /* Create the frame indicator window */
-  s->frame_window = XCreateSimpleWindow (dpy, s->root, 1, 1, 1, 1, defaults.bar_border_width, 
-					 s->fg_color, s->bg_color);
-
-  /* Create the help window */
-  s->help_window = XCreateSimpleWindow (dpy, s->root, 0, 0, s->root_attr.width,
-					s->root_attr.height, 0, s->fg_color, s->bg_color);
-  XSelectInput (dpy, s->help_window, KeyPressMask);
-
-  XSync (dpy, 0);
-}
-
 static void
 free_screen (rp_screen *s)
 {
@@ -788,8 +681,6 @@ free_screen (rp_screen *s)
   XFreeGC (dpy, s->normal_gc);
 
   free (s->display_string);
-
-  numset_free (s->frames_numset);
 }
 
 void
@@ -797,10 +688,14 @@ clean_up ()
 {
   int i;
 
-  free_keybindings ();
+#ifdef HAVE_READLINE_HISTORY_H
+  history_save ();
+#endif /* HAVE_READLINE_HISTORY_H */
+
+  free_keymaps ();
   free_aliases ();
   free_bar ();
-  free_history ();
+  free_groups ();
 
   free_window_stuff ();
   
@@ -810,6 +705,11 @@ clean_up ()
     }
   free (screens);
 
+  /* Free the global frame numset shared by all screens. */
+  free (rp_frame_numset);
+
+  free_xinerama();
+
   XFreeFont (dpy, defaults.font);
   free (defaults.window_fmt);
 
@@ -817,14 +717,3 @@ clean_up ()
   XCloseDisplay (dpy);
 }
 
-/* Given a root window, return the rp_screen struct */
-rp_screen *
-find_screen (Window w)
-{
-  int i;
-
-  for (i=0; i<num_screens; i++)
-    if (screens[i].root == w) return &screens[i];
-
-   return NULL;
- }
