@@ -1,6 +1,6 @@
 /* Manage windows, such as Mapping them and making sure the proper key
  * Grabs have been put in place.
- * Copyright (C) 2000, 2001 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
  *
  * This file is part of ratpoison.
  *
@@ -103,23 +103,79 @@ add_unmanaged_window (char *name)
 extern Atom wm_state;
 
 void
-grab_prefix_key (Window w)
+grab_top_level_keys (Window w)
 {
 #ifdef HIDE_MOUSE
   XGrabKey(dpy, AnyKey, AnyModifier, w, True, 
 	   GrabModeAsync, GrabModeAsync);
 #else
-  grab_key (XKeysymToKeycode (dpy, prefix_key.sym), prefix_key.state, w);
+  rp_keymap *map = find_keymap (TOP_KEYMAP);
+  int i;
+
+  if (map == NULL)
+    {
+      PRINT_ERROR (("Unable to find " TOP_KEYMAP " keymap\n"));
+      return;
+    }
+
+  PRINT_DEBUG(("grabbing top level key\n"));
+  for (i=0; i<map->actions_last; i++)
+    {
+      PRINT_DEBUG(("%d\n", i));
+      grab_key (map->actions[i].key, map->actions[i].state, w);
+    }
 #endif
 }
 
 void
-ungrab_prefix_key (Window w)
+ungrab_top_level_keys (Window w)
 {
-#ifdef HIDE_MOUSE
-#else
-  XUngrabKey(dpy, XKeysymToKeycode (dpy, prefix_key.sym), AnyModifier, w);
-#endif
+  KeySym keysym, upper, lower;
+  rp_keymap *map;
+  int i;
+
+  map = find_keymap (TOP_KEYMAP);
+
+  if (map == NULL)
+    {
+      PRINT_ERROR (("Unable to find " TOP_KEYMAP " keymap\n"));
+      return;
+    }
+
+  /* Make sure we ungrab the right sym. */
+
+  for (i=0; i<map->actions_last; i++)
+    {
+      PRINT_DEBUG(("%d\n", i));
+      keysym = map->actions[i].key;
+      XConvertCase (keysym, &lower, &upper);
+      keysym = lower;
+      XUngrabKey(dpy, XKeysymToKeycode (dpy, keysym), AnyModifier, w);
+    }
+}
+
+void
+ungrab_keys_all_wins ()
+{
+  rp_window *cur;
+
+  /* Remove the grab on the current prefix key */
+  list_for_each_entry (cur, &rp_mapped_window, node)
+    {
+      ungrab_top_level_keys (cur->w);
+    }
+}
+
+void
+grab_keys_all_wins ()
+{
+  rp_window *cur;
+
+  /* Remove the grab on the current prefix key */
+  list_for_each_entry (cur, &rp_mapped_window, node)
+    {
+      grab_top_level_keys (cur->w);
+    }
 }
 
 rp_screen*
@@ -129,7 +185,7 @@ current_screen ()
 
   for (i=0; i<num_screens; i++)
     {
-      if (screens[i].screen_num == rp_current_screen)
+      if (screens[i].xine_screen_num == rp_current_screen)
 	return &screens[i];
     }
   
@@ -347,7 +403,10 @@ void
 unmanage (rp_window *w)
 {
   numset_release (rp_window_numset, w->number);
+
   list_del (&w->node);
+  groups_del_window (w);
+
   free_window (w);  
 
 #ifdef AUTO_CLOSE
@@ -382,14 +441,26 @@ scanwins(rp_screen *s)
   for (i = 0; i < nwins; i++) 
     {
       XGetWindowAttributes(dpy, wins[i], &attr);
-      if (wins[i] == s->bar_window 
-	  || wins[i] == s->key_window 
-	  || wins[i] == s->input_window
-	  || wins[i] == s->frame_window
-	  || wins[i] == s->help_window
+      if (is_rp_window_for_screen(wins[i], s)
 	  || attr.override_redirect == True
 	  || unmanaged_window (wins[i])) continue;
 
+      /* FIXME - with this code, windows which are entirely off-screen
+       * when RP starts won't ever be managed when Xinerama is enabled.
+       */
+      { 
+	XWindowAttributes root_attr;
+
+	XGetWindowAttributes (dpy, s->root, &root_attr);
+      PRINT_DEBUG (("attrs: %d %d %d %d %d %d\n", root_attr.x, root_attr.y, 
+		    s->left, s->top, s->left + s->width, s->top + s->height));}
+
+      if (rp_have_xinerama
+	  && ((attr.x > s->left + s->width)
+               || (attr.x < s->left)
+	       || (attr.y > s->top + s->height)
+	       || (attr.y < s->top))) continue;
+
       win = add_to_window_list (s, wins[i]);
 
       PRINT_DEBUG (("map_state: %s\n", 
@@ -668,7 +739,7 @@ maximize (rp_window *win)
 
 
   /* Actually do the maximizing. */
-  XMoveResizeWindow (dpy, win->w, win->x, win->y, win->width, win->height);
+  XMoveResizeWindow (dpy, win->w, win->scr->left + win->x, win->scr->top + win->y, win->width, win->height);
   XSetWindowBorderWidth (dpy, win->w, win->border);
 
   XSync (dpy, False);
@@ -693,7 +764,7 @@ force_maximize (rp_window *win)
      problem. */
   if (win->hints->flags & PResizeInc)
     {
-      XMoveResizeWindow (dpy, win->w, win->x, win->y,
+      XMoveResizeWindow (dpy, win->w, win->scr->left + win->x, win->scr->top + win->y,
 			 win->width + win->hints->width_inc, 
 			 win->height + win->hints->height_inc);
     }
@@ -702,8 +773,10 @@ force_maximize (rp_window *win)
       XResizeWindow (dpy, win->w, win->width + 1, win->height + 1);
     }
 
+  XSync (dpy, False);
+
   /* Resize the window to its proper maximum size. */
-  XMoveResizeWindow (dpy, win->w, win->x, win->y, win->width, win->height);
+  XMoveResizeWindow (dpy, win->w, win->scr->left + win->x, win->scr->top + win->y, win->width, win->height);
   XSetWindowBorderWidth (dpy, win->w, win->border);
 
   XSync (dpy, False);
@@ -718,12 +791,15 @@ map_window (rp_window *win)
   /* Fill in the necessary data about the window */
   update_window_information (win);
   win->number = numset_request (rp_window_numset);
-  grab_prefix_key (win->w);
+  grab_top_level_keys (win->w);
 
   /* Put win in the mapped window list */
   list_del (&win->node);
   insert_into_list (win, &rp_mapped_window); 
 
+  /* Update all groups. */
+  groups_map_window (win);
+
   /* The window has never been accessed since it was brought back from
      the Withdrawn state. */
   win->last_access = 0;
@@ -801,11 +877,17 @@ withdraw_window (rp_window *win)
 
   /* Give back the window number. the window will get another one,
      if it is remapped. */
+  if (win->number == -1)
+    PRINT_ERROR(("Attempting to withdraw '%s' with number -1!\n", window_name(win)));
+
   numset_release (rp_window_numset, win->number);
   win->number = -1;
 
   list_move_tail(&win->node, &rp_unmapped_window);
 
+  /* Update the groups. */
+  groups_unmap_window (win);
+
   ignore_badwindow++;
 
   XRemoveFromSaveSet (dpy, win->w);
