@@ -1,5 +1,5 @@
 /* handles the handing out of and uniqueness of window numbers.
- * Copyright (C) 2000, 2001 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
  *
  * This file is part of ratpoison.
  *
@@ -71,6 +71,8 @@ numset_find_empty_cell (struct numset *ns)
 int
 numset_add_num (struct numset *ns, int n)
 {
+  PRINT_DEBUG(("ns=%p add_num %d\n", ns, n));
+
   if (numset_num_is_taken (ns, n)) 
     return 0; /* failed. */
   
@@ -84,12 +86,14 @@ int
 numset_request (struct numset *ns)
 {
   int i;
-  
+
   /* look for a unique number, and add it to the list of taken
      numbers. */
   i = 0;
   while (!numset_add_num (ns, i)) i++;
 
+  PRINT_DEBUG(("ns=%p request got %d\n", ns, i));
+
   return i;
 }
 
@@ -100,6 +104,11 @@ numset_release (struct numset *ns, int n)
 {
   int i;
 
+  PRINT_DEBUG(("ns=%p release %d\n", ns, n));
+
+  if (n < 0)
+    PRINT_ERROR(("ns=%p Attempt to release %d!", ns, n));
+
   for (i=0; i<ns->num_taken; i++)
     {
       if (ns->numbers_taken[i] == n) 
