@@ -1,5 +1,5 @@
 /* Standard header for ratpoison.
- * Copyright (C) 2000, 2001 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
  *
  * This file is part of ratpoison.
  *
@@ -30,6 +30,12 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <X11/Xlib.h>
+#include <fcntl.h>
+
+/* Some systems don't define the close-on-exec flag in fcntl.h */
+#ifndef FD_CLOEXEC
+# define FD_CLOEXEC 1
+#endif
 
 /* Helper macro for error and debug reporting. */
 #define PRINT_LINE(type) printf (PACKAGE ":%s:%d: %s: ",__FILE__,  __LINE__, #type)
@@ -58,6 +64,7 @@ extern XGCValues gv;
 #include "conf.h"
 
 #include "data.h"
+#include "globals.h"
 #include "manage.h"
 #include "window.h"
 #include "bar.h"
@@ -70,10 +77,19 @@ extern XGCValues gv;
 #include "split.h"
 #include "frame.h"
 #include "screen.h"
+#include "group.h"
+#include "editor.h"
+#ifdef HAVE_READLINE_HISTORY_H
+#include "history.h"
+#endif /* HAVE_READLINE_HISTORY_H */
+#include "completions.h"
+#include "hook.h"
+#include "xinerama.h"
 
 void clean_up ();
 rp_screen *find_screen (Window w);
 
+void set_close_on_exec (FILE *fd);
 void read_rc_file (FILE *file);
 
 void fatal (const char *msg);
@@ -82,5 +98,6 @@ void *xrealloc (void *ptr, size_t size);
 char *xstrdup (char *s);
 char *xsprintf (char *fmt, ...);
 char *xvsprintf (char *fmt, va_list ap);
+int str_comp (char *s1, char *s2, int len);
 
 #endif /* ! _RATPOISON_H */
