@@ -1,5 +1,5 @@
 /* Function prototypes for handling string buffers.
- * Copyright (C) 2000, 2001 Shawn Betts
+ * Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
  *
  * This file is part of ratpoison.
  *
@@ -30,6 +30,9 @@ sbuf
   char *data;
   size_t len;
   size_t maxsz;
+
+  /* sbuf can exist in a list. */
+  struct list_head node;
 };
 
 struct sbuf *sbuf_new (size_t initsz);
