@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2003 Shawn Betts
+/* Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
  *
  * This file is part of ratpoison.
  *
@@ -30,7 +30,13 @@ int screen_width (rp_screen *s);
 
 struct list_head *screen_copy_frameset (rp_screen *s);
 void screen_restore_frameset (rp_screen *s, struct list_head *head);
+void screen_free_nums (rp_screen *s);
 void frameset_free (struct list_head *head);
 rp_frame *screen_get_frame (rp_screen *s, int frame_num);
 
+void init_screens (int screen_arg, int screen_num);
+
+int is_rp_window_for_screen (Window w, rp_screen *s);
+
+
 #endif
