@@ -1,4 +1,4 @@
-/* Copyright (C) 2000, 2001 Shawn Betts
+/* Copyright (C) 2000, 2001, 2002, 2003 Shawn Betts
  *
  * This file is part of ratpoison.
  *
@@ -29,6 +29,8 @@
 #define VERTICALLY 0
 #define HORIZONTALLY 1
 
+static rp_screen *frames_screen(rp_frame *);
+
 static void
 update_last_access (rp_frame *frame)
 {
@@ -81,8 +83,8 @@ cleanup_frame (rp_frame *frame)
 #ifdef MAXSIZE_WINDOWS_ARE_TRANSIENTS
   if (!win->transient
       && !(win->hints->flags & PMaxSize
-	   && win->hints->max_width < win->scr->root_attr.width
-	   && win->hints->max_height < win->scr->root_attr.height))
+	   && win->hints->max_width < win->scr->width
+	   && win->hints->max_height < win->scr->height))
 #else
   if (!win->transient)
 #endif
@@ -99,6 +101,11 @@ set_frames_window (rp_frame *frame, rp_window *win)
     {
       frame->win_number = win->number;
       win->frame_number = frame->number;
+      
+      /* We need to make sure that win and frame are on the same screen,
+       * since with Xinerama, windows can move from one screen to another.
+       */
+      win->scr = frames_screen(frame);
     }
   else
     {
@@ -187,18 +194,24 @@ init_frame_list (rp_screen *screen)
 }
 
 rp_frame *
-find_last_frame (rp_screen *s)
+find_last_frame ()
 {
   rp_frame *cur, *last = NULL;
   int last_access = -1;
+  int i;
 
-  list_for_each_entry (cur, &s->frames, node)
+  for (i=0; i<num_screens; i++)
     {
-      if (cur->number != s->current_frame
-	  && cur->last_access > last_access)
+      rp_screen *s = &screens[i];
+
+      list_for_each_entry (cur, &s->frames, node)
 	{
-	  last_access = cur->last_access;
-	  last = cur;
+	  if (cur->number != current_screen()->current_frame
+	      && cur->last_access > last_access)
+	    {
+	      last_access = cur->last_access;
+	      last = cur;
+	    }
 	}
     }
 
@@ -267,24 +280,27 @@ find_window_for_frame (rp_frame *frame)
 {
   rp_screen *s = frames_screen (frame);
   int last_access = 0;
-  rp_window *most_recent = NULL;
-  rp_window *cur;
+  rp_window_elem *most_recent = NULL;
+  rp_window_elem *cur;
 
-  list_for_each_entry (cur, &rp_mapped_window, node)
+  list_for_each_entry (cur, &rp_current_group->mapped_windows, node)
     {
-      if (cur->scr == s
-	  && cur != current_window()
-	  && !find_windows_frame (cur)
-	  && cur->last_access >= last_access
-	  && window_fits_in_frame (cur, frame)
-	  && cur->frame_number == EMPTY)
+      if ((cur->win->scr == s || rp_have_xinerama)
+	  && cur->win != current_window()
+	  && !find_windows_frame (cur->win)
+	  && cur->win->last_access >= last_access
+	  && window_fits_in_frame (cur->win, frame)
+	  && cur->win->frame_number == EMPTY)
 	{
 	  most_recent = cur;
-	  last_access = cur->last_access;
+	  last_access = cur->win->last_access;
 	}
     }
 
-  return most_recent;
+  if (most_recent)
+    return most_recent->win;
+
+  return NULL;
 }
 
 /* Splits the frame in 2. if way is 0 then split vertically otherwise
@@ -381,7 +397,7 @@ remove_all_splits ()
   /* Hide all the windows not in the current frame. */
   list_for_each_entry (win, &rp_mapped_window, node)
     {
-      if (win->frame_number != s->current_frame)
+      if (win->frame_number != s->current_frame && win->scr == s)
 	hide_window (win);
     }
 
@@ -837,7 +853,7 @@ set_active_frame (rp_frame *frame)
   s->current_frame = frame->number;
 
   /* If frame->win == NULL, then rp_current_screen is not updated. */
-  rp_current_screen = s->screen_num;
+  rp_current_screen = s->xine_screen_num;
 
   update_bar (s);
 
@@ -846,14 +862,17 @@ set_active_frame (rp_frame *frame)
       || s != old_s)
     {
       show_frame_indicator();
+
+      /* run the frame switch hook. We call it in here because this is
+	 when a frame switch ACTUALLY (for sure) happens. */
+      hook_run (&rp_switch_frame_hook);
     }
 
   /* If the frame has no window to give focus to, give the key window
      focus. */
   if(frame->win_number == EMPTY)
     {
-      XSetInputFocus (dpy, s->key_window,
-		      RevertToPointerRoot, CurrentTime);
+      set_window_focus (s->key_window);
     }
 }
 
@@ -876,19 +895,21 @@ blank_frame (rp_frame *frame)
   update_bar (s);
 
   /* Give the key window focus. */
-  XSetInputFocus (dpy, current_screen()->key_window,
-		  RevertToPointerRoot, CurrentTime);
+  set_window_focus (current_screen()->key_window);
 }
 
 void
 hide_frame_indicator ()
 {
-  XUnmapWindow (dpy, current_screen()->frame_window);
+  int i;
+  for (i=0; i<num_screens; i++)
+    XUnmapWindow (dpy, screens[i].frame_window);
 }
 
 void
 show_frame_indicator ()
 {
+  hide_frame_indicator ();
   show_frame_message (MESSAGE_FRAME_STRING);
   alarm (defaults.frame_indicator_timeout);
 }
@@ -905,9 +926,14 @@ show_frame_message (char *msg)
   width = defaults.bar_x_padding * 2 + XTextWidth (defaults.font, msg, strlen (msg));
   height = (FONT_HEIGHT (defaults.font) + defaults.bar_y_padding * 2);
 
+  /* We don't want another frame indicator to be displayed on another
+   * screen at the same time, so we hide it before bringing it back again.
+   */
+  hide_frame_indicator ();
+
   XMoveResizeWindow (dpy, s->frame_window,
-		     frame->x + frame->width / 2 - width / 2,
-		     frame->y + frame->height / 2 - height / 2,
+		     s->left + frame->x + frame->width / 2 - width / 2,
+		     s->top + frame->y + frame->height / 2 - height / 2,
 		     width, height);
 
   XMapRaised (dpy, s->frame_window);
@@ -993,14 +1019,20 @@ find_frame_right (rp_frame *frame)
 }
 
 rp_frame *
-find_frame_number (rp_screen *s, int num)
+find_frame_number (int num)
 {
+  int i;
   rp_frame *cur;
 
-  list_for_each_entry (cur, &s->frames, node)
+  for (i=0; i<num_screens; i++)
     {
-      if (cur->number == num)
-	return cur;
+      rp_screen *s = &screens[i];
+  
+      list_for_each_entry (cur, &s->frames, node)
+	{
+	  if (cur->number == num)
+	    return cur;
+	}
     }
 
   return NULL;
